/** simplecom - A simple communication protocol for two peers in an ethernet segment. */
#ifndef _SIMPLECOM_H
#define _SIMPLECOM_H

#ifndef __USE_MISC
#  define __USE_MISC
#endif

#include <net/if.h> /* IFNAMSIZ */
#include <linux/if_ether.h> /* ETH_ALEN */
#include <linux/types.h>
#include <net/ethernet.h> /* ether_header */
#include <netpacket/packet.h> /* sockaddr_ll */

/** custom ethertype used by simplecom packets, should
 * not conflict with others (see linux/if_ether.h) */
#define ETH_P_SCOM	0xff23

/** upper limit for interface's MTU
 * (simplecom packets won't grow larger than this) */
#define MAX_MTU		1500

/** interface meta information used when creating packets */
struct interface {
	char name[IFNAMSIZ];		/**< interface name */
	int index;			/**< interface index */
	int sockfd;			/**< open socket (bound) */
	char hwaddr[ETH_ALEN];		/**< interface's hwaddr */
	int mtu;			/**< interface's MTU */
	__u32 seqno;			/**< current sending seqno */
	struct sockaddr_ll send_sa;	/**< prepared sender sockaddr */
	struct ether_header eh;		/**< prepared ethernet header for sending */
	__u8 proto;			/**< protocol type */
};

/** simplecom packet header */
struct scom_hdr {
	__u32 seqno;		/**< sequence number, used for fragments */
	__u8 flags;		/**< packet flags, also useful for fragmentation only */
	__u8 proto;		/**< next layer protocol */
	__u16 pktlen;		/**< packet payload length */
} __attribute__((__packed__));

/** valid simplecom flag values (may be combined by bitwise OR'ing them) */
#define SCFL_NONE	0x00	/**< no flag set at all (bogus) */
#define SCFL_MF		0x01	/**< more fragments follow */
#define SCFL_FRAG	0x02	/**< packet is a fragment */

/** higher layer protocol */
enum simplecom_proto {
	SCPROTO_NONE = 0,	/**< no higher layer protocol (nonsense) */
	SCPROTO_DSLCONF = 1,	/**< payload is dslconf paket */
};

/** fetch meta information for a given interface */
struct interface *init_interface(const char *, __u8);

/** free struct interface object */
void exit_interface(struct interface *);

/** low level send and receive routines (used by dslconf internally) */
ssize_t send_packet(struct interface, void *, __u32);
ssize_t recv_packet(struct interface, void **, __u32 *);

#endif /* _SIMPLECOM_H */
