/* sqfsinfo - extract squashfs image version and print fstype string
 * Copyright (C) 2012  Phil Sutter <phil.sutter@viprinet.com>
 *
 * Sqfsinfo is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your option)
 * any later version.
 *
 * Sqfsinfo is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with sqfsinfo.  If not, see <http://www.gnu.org/licenses/>.
 */
#include <fcntl.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>

static void usage(void)
{
#if 0
	/* TODO: implement this some time */
	puts("Usage:\n"
	     "sqfsinfo [options] <imagename>\n\n"
	     "Options:\n"
	     "-c\tcheck (return 0 if image is squashfs, 1 otherwise)\n"
	     "-e\tendianness (print \"big\" or \"little\")\n"
	     "-v\tversion (print the version)\n"
	     "-m\tmajor version (print only the major version part)");
#else
	fputs("Usage: sqfsinfo <imagename>\n\n"
	     "Prints \"squashfs\" or \"squashfs3\", "
	     "according to SquashFS major\n", stderr);
#endif
}

int main(int argc, char **argv)
{
	int fd;
	char magic[5];
	char version;

	if (argc != 2) {
		usage();
		return 1;
	}

	if ((fd = open(argv[1], O_RDONLY)) == -1) {
		perror("open()");
		return 1;
	}

	if (read(fd, magic, 4) != 4) {
		fputs("short read!\n", stderr);
		return 1;
	}

	magic[4] = 0;
	if (strcmp(magic, "sqsh") && strcmp(magic, "hsqs")) {
		fputs("not a squashfs image!\n", stderr);
		return 1;
	}

	/* According to the specs, squashfs version is a big endian
	 * short at offset 28. But since we are only interested in the
	 * major (3 or 4), just read the char at offset 28 and skip
	 * conversion to host byte order. */

	if (lseek(fd, 28, SEEK_SET) != 28) {
		perror("lseek()");
		return 1;
	}

	if (read(fd, &version, 1) != 1) {
		fputs("short read!\n", stderr);
		return 1;
	}

	switch(version) {
	case 4:
		puts("squashfs");
		break;
	case 3:
		puts("squashfs3");
		break;
	default:
		fprintf(stderr, "unknown squashfs major (%d)!\n", version);
		return 1;
	}

	close(fd);
	return 0;
}
